"""add Show.etvdb_id

Revision ID: 376061fc33e
Revises: 1d11afd9a90
Create Date: 2015-09-23 15:34:36.434877

"""

# revision identifiers, used by Alembic.
revision = '376061fc33e'
down_revision = '1d11afd9a90'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('show', sa.Column('etvdb_id', sa.String(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('release_monitor', sa.Column('torrent_cached', sa.BOOLEAN(), nullable=True))
    ### end Alembic commands ###
